#!/usr/bin/python

import sys
import subprocess
import os
import yaml
import socket
import struct
try:
    import pygtk
    pygtk.require('2.0')
    import gtk
    import gobject
except:
    print "GTK bindings not found"
    
    
ARTNET_IP='192.168.22.21'
ARTNET_PORT=0x1936

class artnet:
    def __init__(self):
        self.sock = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
        self.seq = 0
    def send_packet(self,data,ip='255.255.255.255',port=0x1936):
        self.sock.sendto(self.artnet_packet(data),(ip,port))
        
    def artnet_packet(self,data):
        self.seq+=1
        if self.seq > 255:
            self.seq = 0
        out = struct.pack('!8s8BH','Art-Net\0x00',0x00,0x50,0,14,self.seq,0,0,0,
                          len(data))
        for i in range(0,len(data)):
            out+=struct.pack('B',data[i])
        return out

class showgui:
    def __init__(self):
        # Variable initialization
        self.artnet = artnet()
        
        self.data=[]
        self.target=[]
        self.orig=[]
        self.seq = 0
        for i in range(0,512):
            self.data.append(0)
            self.target.append(0)
            self.orig.append(0)
        self.timer = None
        self.actual_file = None
        
        # Loading configuration     
        self.config = yaml.load(open('show.yaml','r'))
        print self.config
        # Loading GUI
        self.uifile = "simple_show.ui"
        self.windowname = "show"
        self.wTree = gtk.Builder();
        self.wTree.add_from_file(self.uifile)
        assert (self.wTree)
        self.window = self.wTree.get_object(self.windowname)
        assert (self.window)
        self.buttons=[];
        for i in range(0,10):
            print ("Looking up button"+str(i))
            self.buttons.append(self.wTree.get_object("button"+str(i)))
        self.state_text = self.wTree.get_object("state")
        self.progress = self.wTree.get_object("progress")
        
        dic = {"show_destroy"       : self.destroy,
               "clicked"           : self.clicked}        
        self.wTree.connect_signals(dic)

        self.window.show()


    def send_packet(self):
        print "Sending packet"
        self.artnet.send_packet(self.data, ARTNET_IP, ARTNET_PORT)

    def load_target(self,name):
        print self.config[name]
        for i in range (0, 512):
            if (i+1) in self.config[name]:
                self.target[i]=int(self.config[name][i+1]*2.55)
                print "Loaded value ", self.target[i], "@", i 
            else:
                self.target[i]=0


########################################
# Call 
########################################
    
    def clicked(self,widget,data=None):
        print "Clicked: ", widget.get_label()
        if widget == self.buttons[1]:
            if self.trans('dark'):
                self.state_text.set_label(widget.get_label())
        elif widget == self.buttons[2]:
            if self.trans('work'):
                self.state_text.set_label(widget.get_label())
        elif widget == self.buttons[4]:
            if self.trans('seq1'):
                self.state_text.set_label(widget.get_label())
        elif widget == self.buttons[5]:
            if self.trans('seq2'):
                self.state_text.set_label(widget.get_label())
        elif widget == self.buttons[6]:
            if self.trans('seq3'):
                self.state_text.set_label(widget.get_label())
        elif widget == self.buttons[7]:
            if self.trans('seq4'):
                self.state_text.set_label(widget.get_label())
        elif widget == self.buttons[8]:
            if self.trans('seq5'):
                self.state_text.set_label(widget.get_label())
        elif widget == self.buttons[9]:
            if self.trans('seq6'):
                self.state_text.set_label(widget.get_label())        
        elif widget == self.buttons[3]:
            self.flash('smoke',1000);
        elif widget == self.buttons[0]:
            self.flash('flash',80);
            
    def trans(self,file2):
        if (self.timer != None):
            return False
        self.load_target(file2)
        if self.actual_file and not self.actual_file == file2:
            print self.actual_file," -> ", file2
            self.progress.set_fraction(0.0)
            self.timer = gobject.timeout_add(30,self.tick,0.01)
        else:
            print "|-> ", file2
            self.progress.set_fraction(1.0)
            self.copy_data(self.target)
            self.copy_data(self.data, self.orig)
            self.send_packet()
        self.actual_file = file2;
        return True
        
        
    def tick(self,delta):
        val = self.progress.get_fraction() + delta
        print val
        if (val>1.0):
            self.progress.set_fraction(1.0)
            self.copy_data(self.target)
            self.copy_data(self.data, self.orig)
            self.send_packet()
            self.timer = None
            return False
        self.progress.set_fraction(val)
        self.comp_data(self.orig,self.target,val)
        self.send_packet()
        return True
        
    def flash(self,file,time):        
        if (self.timer != None):
            return False
        print "Flashing ", file, " in ",self.actual_file," for ",str(time)," s"
        self.load_target(file)
        self.add_data(self.target)
        self.send_packet()
        self.timer = gobject.timeout_add(time,self.reset)
        
    def reset(self):
        print "Reseting"
        self.copy_data(self.orig, self.data)
        self.send_packet()
        self.timer = None
        return False
         
    def copy_data(self,data,target=None):
        if (not target):
            target = self.data
        for i in range (0, 512):
            target[i]=data[i]
            
    def add_data(self,data):
        for i in range (0, 512):
            if data[i] > self.data[i]:
                self.data[i] = data[i]
                
    def comp_data(self,old,new,val):
        for i in range (0, 512):
            self.data[i]=int(old[i]*(1-val)+new[i]*val)
            if (self.data[i] > 0):
                print i, ": ", self.data[i]
        
    def destroy(self,widget):
        gtk.main_quit()

gui = showgui()
gtk.main()
